#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include "serialGLCD.h"
#include <stdlib.h>

#define SUS 	PB7
#define STANGA 	PB6
#define DREAPTA PB5
#define JOS 	PB4

#define LUNGIME 128
#define LATIME 	64
#define START_X	64
#define START_Y	32

#define SIZE_INITIAL	14

#define DIRECTIE_SUS		0
#define DIRECTIE_JOS		1
#define DIRECTIE_STANGA		2
#define DIRECTIE_DREAPTA	3

typedef struct {

	int x, y;
} punct;

punct snake[264], food;
int size_snake;
int directie;

void clear_snake() {
	int i;
	for (i = 0; i < size_snake; ++i)
		togglePixel(snake[i].x, snake[i].y, 0);
}
void clear_food() {
	togglePixel(food.x, food.y, 0);
}
void show_food() {
	togglePixel(food.x, food.y, 1);
}
void show_snake() {
	int i;
	for (i = 0; i < size_snake; ++i)
		togglePixel(snake[i].x, snake[i].y, 1);
}
int check_point(punct p) {
	int i;
	
	if (p.x > LUNGIME - 2 || p.x < 1 || p.y > LATIME - 2 || p.y < 1)
		return 0;
	for (i = 0; i < size_snake; ++i)
		if (p.x == snake[i].x && p.y == snake[i].y)
			return 0;
	return 1;
}

punct generate_food() {
	punct food;
	
	while(1) {
		food.x = rand() % LATIME;
		food.y = rand() % LUNGIME;
	if (check_point(food) == 1)
		break;
	}
	return food;
}


void init_snake() {
	size_snake = SIZE_INITIAL;
	directie = DIRECTIE_SUS;
	int i;
	for (i = 0; i < LUNGIME; ++i) {
		togglePixel(i, 0, 1);
		togglePixel(i, LATIME - 1, 1);
		togglePixel(0, i, 1);
		togglePixel(LUNGIME - 1, i, 1);
	}
	for (i = 0; i < size_snake; ++i) {
		snake[i].x = START_X;
		snake[i].y = START_Y + i;
	}
	
	show_snake();
	
	food = generate_food();
	
	show_food();
	
}



void init() {
	clearLCD();
	
	DDRB &= ~(1 << SUS);
	PORTB |= (1 << SUS);
	
	DDRB &= ~(1 << JOS);
	PORTB |= (1 << JOS);
	
	DDRB &= ~(1 << STANGA);
	PORTB |= (1 << STANGA);
	
	DDRB &= ~(1 << DREAPTA);
	PORTB |= (1 << DREAPTA);
}

void show_unu(int i, int x) {
	togglePixel(x - 1, 50, i);
	
	togglePixel(x, 49, i);
	togglePixel(x - 1, 49, i);
	
	togglePixel(x - 1, 48, i);

	togglePixel(x - 1, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}

void show_doi(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x - 2, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}

void show_trei(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x - 2, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x - 2, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}

void show_patru(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x, 49, i);
	togglePixel(x - 2, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x - 2, 47, i);
	
	togglePixel(x - 2, 46, i);
}
void show_cinci(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x - 2, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}
void show_sase(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x, 47, i);
	togglePixel(x - 2, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}
void show_sapte(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x - 2, 49, i);
	
	togglePixel(x - 2, 48, i);
	
	togglePixel(x - 2, 47, i);

	togglePixel(x - 2, 46, i);
}
void show_opt(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x, 49, i);
	togglePixel(x - 2, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x, 47, i);
	togglePixel(x - 2, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}
void show_noua(int i, int x) {
	togglePixel(x, 50, i);
	togglePixel(x - 1, 50, i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x, 49, i);
	togglePixel(x - 2, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 1, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x - 2, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}
void show_zero(int i, int x) {
	togglePixel(x, 50,  i);
	togglePixel(x - 1,50,  i);
	togglePixel(x - 2, 50, i);
	
	togglePixel(x, 49, i);
	togglePixel(x - 2, 49, i);
	
	togglePixel(x, 48, i);
	togglePixel(x - 2, 48, i);
	
	togglePixel(x, 47, i);
	togglePixel(x - 2, 47, i);
	
	togglePixel(x, 46, i);
	togglePixel(x - 1, 46, i);
	togglePixel(x - 2, 46, i);
}
void show_score(int i) {
	togglePixel(120, 50, i);
	togglePixel(119, 50, i);
	togglePixel(118, 50, i);
	
	togglePixel(116, 50, i);
	togglePixel(115, 50, i);
	togglePixel(114, 50, i);
	
	togglePixel(112, 50, i);
	togglePixel(111, 50, i);
	togglePixel(110, 50, i);
	
	togglePixel(108, 50, i);
	togglePixel(107, 50, i);
	togglePixel(106, 50, i);
	/////////////////////////////
	togglePixel(120, 49, i);
	
	togglePixel(116, 49, i);
	
	togglePixel(112, 49, i);
	togglePixel(110, 49, i);
	
	togglePixel(108, 49, i);
	togglePixel(106, 49, i);
	/////////////////////////
	togglePixel(120, 48, i);
	togglePixel(119, 48, i);
	togglePixel(118, 48, i);
	
	togglePixel(116, 48, i);
	
	togglePixel(112, 48, i);
	togglePixel(110, 48, i);
	
	togglePixel(108, 48, i);
	togglePixel(107, 48, i);
	togglePixel(106, 48, i);
	////////////////////////
	togglePixel(118, 47, i);
	
	togglePixel(116, 47, i);
	
	togglePixel(112, 47, i);
	togglePixel(110, 47, i);
	
	togglePixel(108, 47, i);
	togglePixel(106, 47, i);
	///////////////////////
	togglePixel(120, 46, i);
	togglePixel(119, 46, i);
	togglePixel(118, 46, i);
	
	togglePixel(116, 46, i);
	togglePixel(115, 46, i);
	togglePixel(114, 46, i);
	
	togglePixel(112, 46, i);
	togglePixel(111, 46, i);
	togglePixel(110, 46, i);
	
	togglePixel(108, 46, i);
	togglePixel(105, 46, i);
	int score = size_snake - SIZE_INITIAL;
	int poz = 70;
	do {
		int cifra = score % 10;
		switch (cifra){
			case 0:
				show_zero(i, poz);
				break;
			case 1:
				show_unu(i, poz);
				break;
			case 2:
				show_doi(i, poz);
				break;
			case 3:
				show_trei(i, poz);
				break;
			case 4:
				show_patru(i, poz);
				break;
			case 5:
				show_cinci(i, poz);
				break;
			case 6:
				show_sase(i, poz);
				break;
			case 7:
				show_sapte(i, poz);
				break;
			case 8:
				show_opt(i, poz);
				break;
			case 9:
				show_noua(i, poz);
				break;
			default:
				show_zero(i, poz);
				break;
		}
		poz += 4;
		score /= 10;
	} while(score > 0);
}

void play() {
	int i;
	
	while(1) {
	
		_delay_ms(100);
		if ((PINB & (1 << SUS)) == 0 && directie != DIRECTIE_JOS) {
			directie = DIRECTIE_SUS;
		}
		else if ((PINB & (1 << JOS)) == 0  && directie != DIRECTIE_SUS) {
			directie = DIRECTIE_JOS;
		}
		else if ((PINB & (1 << STANGA)) == 0 && directie != DIRECTIE_DREAPTA) {
			directie = DIRECTIE_STANGA;
		}
		else if ((PINB & (1 << DREAPTA)) == 0 && directie != DIRECTIE_STANGA) {
			directie = DIRECTIE_DREAPTA;
		}
		
		punct nou = snake[size_snake - 1];
		switch(directie) {
			case DIRECTIE_SUS:
				nou.y += 1;
				break;
			case DIRECTIE_JOS:
				nou.y -= 1;
				break;
			case DIRECTIE_STANGA:
				nou.x += 1;
				break;
			default:
				nou.x -= 1;
				break;
		}
		if (!check_point(nou)) {
			clear_snake();
			clear_food();
			show_score(1);
			_delay_ms(1000);
			show_score(0);
			init_snake();
		}
		else {
			clear_snake();
			if (nou.x == food.x && nou.y == food.y) {
				snake[size_snake] = nou;
				size_snake++;
				food = generate_food();
				show_food();
				
			} else {
				for (i = 0; i < size_snake - 1; ++i)
					snake[i] = snake[i + 1];
				snake[size_snake - 1] = nou;
			}
			show_snake();
		}
	}
}

int main(void) {
	init();
	srand(LUNGIME);
	init_snake();
	play();
	return 0;
}
